@@ -68,6 +68,73 @@ struct move_item {
 	obs_scene_t *release_scene_b;
 };
 
+static const struct {
+	enum gs_blend_type src_color;
+	enum gs_blend_type src_alpha;
+	enum gs_blend_type dst_color;
+	enum gs_blend_type dst_alpha;
+	enum gs_blend_op_type op;
+} obs_blend_mode_params[] = {
+	/* clang-format off */
+	// OBS_BLEND_NORMAL
+	{
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_INVSRCALPHA,
+		GS_BLEND_INVSRCALPHA,
+		GS_BLEND_OP_ADD,
+	},
+	// OBS_BLEND_ADDITIVE
+	{
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_OP_ADD,
+	},
+	// OBS_BLEND_SUBTRACT
+	{
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_OP_REVERSE_SUBTRACT,
+	},
+	// OBS_BLEND_SCREEN
+	{
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_INVSRCCOLOR,
+		GS_BLEND_INVSRCALPHA,
+		GS_BLEND_OP_ADD
+	},
+	// OBS_BLEND_MULTIPLY
+	{
+		GS_BLEND_DSTCOLOR,
+		GS_BLEND_DSTALPHA,
+		GS_BLEND_INVSRCALPHA,
+		GS_BLEND_INVSRCALPHA,
+		GS_BLEND_OP_ADD
+	},
+	// OBS_BLEND_LIGHTEN
+	{
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_OP_MAX,
+	},
+	// OBS_BLEND_DARKEN
+	{
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_ONE,
+		GS_BLEND_OP_MIN,
+	},
+	/* clang-format on */
+};
+
 static const char *move_get_name(void *type_data)
 {
 	UNUSED_PARAMETER(type_data);
@@ -352,6 +419,11 @@ static inline bool crop_enabled(const struct obs_sceneitem_crop *crop)
 	return crop->left || crop->right || crop->top || crop->bottom;
 }
 
+static inline bool default_blending_enabled(struct obs_scene_item *item)
+{
+	return obs_sceneitem_get_blending_mode(item) == OBS_BLEND_NORMAL;
+}
+
 static inline bool item_texture_enabled(struct obs_scene_item *item)
 {
 	if (!item)
@@ -359,6 +431,7 @@ static inline bool item_texture_enabled(struct obs_scene_item *item)
 	struct obs_sceneitem_crop crop;
 	obs_sceneitem_get_crop(item, &crop);
 	return crop_enabled(&crop) || scale_filter_enabled(item) ||
+	       !default_blending_enabled(item) ||
 	       (item_is_scene(item) && !obs_sceneitem_is_group(item));
 }
 
@@ -579,7 +652,8 @@ static obs_source_t *obs_frontend_get_transition(const char *name)
 		const char *n =
 			obs_source_get_name(transitions.sources.array[i]);
 		if (n && strcmp(n, name) == 0) {
-			obs_source_t *transition = obs_source_get_ref(transitions.sources.array[i]);
+			obs_source_t *transition = obs_source_get_ref(
+				transitions.sources.array[i]);
 			obs_frontend_source_list_free(&transitions);
 			return transition;
 		}
@@ -1320,7 +1394,15 @@ bool render2_item(struct move_info *move, struct move_item *item)
 		}
 
 		gs_blend_state_push();
-		gs_blend_function(GS_BLEND_ONE, GS_BLEND_INVSRCALPHA);
+
+		enum obs_blending_type blend_type =
+			obs_sceneitem_get_blending_mode(scene_item);
+		gs_blend_function_separate(
+			obs_blend_mode_params[blend_type].src_color,
+			obs_blend_mode_params[blend_type].dst_color,
+			obs_blend_mode_params[blend_type].src_alpha,
+			obs_blend_mode_params[blend_type].dst_alpha);
+		gs_blend_op(obs_blend_mode_params[blend_type].op);
 
 		while (gs_effect_loop(effect, tech))
 			obs_source_draw(tex, 0, 0, 0, 0, 0);
@@ -1658,13 +1740,9 @@ void sceneitem_start_move(obs_sceneitem_t *item, const char *start_move)
 	}
 	if (!filter)
 		return;
-	const char *filter_id = obs_source_get_unversioned_id(filter);
-	if (strcmp(filter_id, MOVE_SOURCE_FILTER_ID) == 0) {
-		move_source_start(obs_obj_get_data(filter));
-	} else if (strcmp(filter_id, MOVE_VALUE_FILTER_ID) == 0 ||
-		   strcmp(filter_id, MOVE_AUDIO_VALUE_FILTER_ID) == 0) {
-		move_value_start(obs_obj_get_data(filter));
-	}
+	
+	if(is_move_filter(obs_source_get_unversioned_id(filter)))
+		move_filter_start(obs_obj_get_data(filter));
 }
 
 static void move_video_render(void *data, gs_effect_t *effect)
@@ -1704,6 +1782,7 @@ static void move_video_render(void *data, gs_effect_t *effect)
 				obs_scene_add(scene_a, move->scene_source_a);
 			struct move_item *item = create_move_item();
 			da_push_back(move->items_a, &item);
+			obs_sceneitem_addref(scene_item);
 			item->item_a = scene_item;
 			item->release_scene_a = scene_a;
 		}
@@ -1734,6 +1813,7 @@ static void move_video_render(void *data, gs_effect_t *effect)
 				da_insert(move->items_a, move->item_pos, &item);
 				move->item_pos++;
 			}
+			obs_sceneitem_addref(scene_item);
 			item->item_b = scene_item;
 			item->release_scene_b = scene_b;
 			da_push_back(move->items_b, &item);
@@ -2480,6 +2560,11 @@ extern struct obs_source_info move_source_filter;
 extern struct obs_source_info move_value_filter;
 extern struct obs_source_info move_audio_value_filter;
 extern struct obs_source_info audio_move_filter;
+extern struct obs_source_info move_action_filter;
+extern struct obs_source_info move_audio_action_filter;
+#ifdef WIN32
+void SetMoveDirectShowFilter(struct obs_source_info *obs_source_info);
+#endif
 
 bool obs_module_load(void)
 {
@@ -2490,5 +2575,12 @@ bool obs_module_load(void)
 	obs_register_source(&move_value_filter);
 	obs_register_source(&move_audio_value_filter);
 	obs_register_source(&audio_move_filter);
+	obs_register_source(&move_action_filter);
+	obs_register_source(&move_audio_action_filter);
+#ifdef WIN32
+	struct obs_source_info move_directshow_filter = {0};
+	SetMoveDirectShowFilter(&move_directshow_filter);
+	obs_register_source(&move_directshow_filter);
+#endif
 	return true;
 }
